@@ -1,6 +1,6 @@
 package DBIx::Class::Async::Row;
 
-$DBIx::Class::Async::Row::VERSION   = '0.60';
+$DBIx::Class::Async::Row::VERSION   = '0.61';
 $DBIx::Class::Async::Row::AUTHORITY = 'cpan:MANWAR';
 
 =head1 NAME
@@ -9,7 +9,7 @@ DBIx::Class::Async::Row - Asynchronous Row object representing a single database
 
 =head1 VERSION
 
-Version 0.60
+Version 0.61
 
 =head1 SYNOPSIS
 
@@ -499,6 +499,75 @@ sub in_storage {
     return $self->{_in_storage} // 0;
 }
 
+=head2 insert_query
+
+    my ($sql, @bind) = $row->insert_query;
+    my $query = $row->insert_query;
+
+Returns the SQL and bind values that would be generated by an insert operation
+without actually executing it. The row must be in_storage(false) state.
+
+    my $row = $schema->resultset('User')->new({
+        name  => 'Alice',
+        email => 'alice@example.com',
+    });
+
+    my ($sql, @bind) = $row->insert_query;
+    # $sql = \'INSERT INTO users (name, email) VALUES (?, ?)'
+    # @bind = ('Alice', 'alice@example.com')
+
+    # Now actually insert
+    $row->insert->get;
+
+=cut
+
+sub insert_query {
+    my $self = shift;
+
+    Carp::croak("insert_query can only be called on rows not yet in storage")
+        if $self->in_storage;
+
+    my $bridge       = $self->{_async_db};
+    my $schema_class = $bridge->{_schema_class};
+
+    unless ($schema_class->can('resultset')) {
+        eval "require $schema_class" or die "insert_query: $@";
+    }
+
+    # Silence warnings
+    local $SIG{__WARN__} = sub {
+        warn @_ unless $_[0] =~ /undetermined_driver|sql_limit_dialect|GenericSubQ/;
+    };
+
+    unless ($bridge->{_metadata_schema}) {
+        $bridge->{_metadata_schema} = $schema_class->connect('dbi:NullP:');
+    }
+
+    my $storage   = $bridge->{_metadata_schema}->storage;
+    my $source    = $bridge->{_metadata_schema}->source($self->{_source_name});
+    my %to_insert = $self->get_columns;
+
+    my ($sql, @bind);
+
+    eval {
+        my $table = $source->from;
+        my @cols  = sort keys %to_insert;
+        my @placeholders = ('?') x @cols;
+
+        my $cols_sql = join(', ', @cols);
+        my $vals_sql = join(', ', @placeholders);
+
+        $sql  = \"INSERT INTO $table ($cols_sql) VALUES ($vals_sql)";
+        @bind = @to_insert{@cols};
+    };
+
+    if ($@) {
+        croak("Failed to generate insert SQL: $@");
+    }
+
+    return wantarray ? ($sql, @bind) : \[ $sql, @bind ];
+}
+
 sub is_column_changed {
     my ($self, $column) = @_;
 
